"""Module with utilities for use within CyIpopt.

Currently contains functions to aid with deprecation within CyIpopt.

"""

import warnings
from functools import wraps

from ipopt_wrapper import Problem


def deprecated_warning(new_name):
    """Decorator that issues a FutureWarning for deprecated functionality.

    Parameters
    ----------
    new_name : :obj:`str`
        The name of the object replacing the deprecated one.

    Returns
    -------
    :obj:`decorator`
        For decorating functions that will soon be deprecated.

    """

    def decorate(func):

        @wraps(func)
        def wrapper(*args, **kwargs):
            if hasattr(func, "__objclass__"):
                what = "method"
                class_name = getattr(func, "__objclass__").__name__
                msg = generate_deprecation_warning_msg(what,
                                                       old_name,
                                                       new_name,
                                                       class_name=class_name)
            else:
                what = "function"
                msg = generate_deprecation_warning_msg(
                    what, old_name, new_name)
            warnings.warn(msg, FutureWarning)
            return func(*args, **kwargs)

        old_name = getattr(func, "__name__")
        return wrapper

    return decorate


def generate_deprecation_warning_msg(what,
                                     old_name,
                                     new_name,
                                     class_name=None):
    """Helper function to create user-friendly deprecation messages.

    Parameters
    ----------
    what : str
        The type of object that is being deprecated. Expected values are
        :str:`class`, :str:`function` and :str:`method`.
    old_name : str
        The name of the object being deprecated.
    new_name : str
        The name of the object replacing the deprecated object.
    class_name : str, optional
        The class name if the object being deprecated is a :obj:`class`.
        Default value is :obj:`None`.

    Returns
    -------
    str
        The nicely formatted informative :obj:`str` to be outputted as the
        warning message.

    Raises
    ------
    ValueError
        If a :arg:`class_name` is supplied but :arg:`what` does not equal
        :str:`"class"`.
    """
    if what == "class" and class_name is not None:
        msg = "Incorrect use of function arguments."
        raise ValueError(msg)
    if class_name is not None:
        class_name_msg = f"in class '{str(class_name)}' "
    else:
        class_name_msg = ""
    msg = (f"The {what} named '{old_name}' {class_name_msg}will soon be "
           f"deprecated in CyIpopt. Please replace all uses and use "
           f"'{new_name}' going forward.")
    return msg



"""Test functionality of CyIpopt with/without optional SciPy dependency.

SciPy is an optional dependency of CyIpopt. CyIpopt needs to function without
SciPy installed, but also needs to provide the :func:`minimize_ipopt` function
which requires SciPy.

"""

import re
import sys

import numpy as np
import pytest

import cyipopt


@pytest.mark.skipif("scipy" in sys.modules,
                    reason="Test only valid if no Scipy available.")
def test_minimize_ipopt_import_error_if_no_scipy():
    """`minimize_ipopt` not callable without SciPy installed."""
    expected_error_msg = re.escape("Install SciPy to use the "
                                   "`minimize_ipopt` function.")
    with pytest.raises(ImportError, match=expected_error_msg):
        cyipopt.minimize_ipopt(None, None)


@pytest.mark.skipif("scipy" not in sys.modules,
                    reason="Test only valid if Scipy available.")
def test_minimize_ipopt_if_scipy():
    """If SciPy is installed `minimize_ipopt` works correctly."""
    from scipy.optimize import rosen, rosen_der
    x0 = [1.3, 0.7, 0.8, 1.9, 1.2]
    res = cyipopt.minimize_ipopt(rosen, x0, jac=rosen_der)
    assert isinstance(res, dict)
    assert np.isclose(res.get("fun"), 0.0)
    assert res.get("status") == 0
    assert res.get("success") is True
    np.testing.assert_allclose(res.get("x"), np.ones(5))


@pytest.mark.skipif("scipy" not in sys.modules,
                    reason="Test only valid if Scipy available.")
def test_minimize_ipopt_nojac_if_scipy():
    """`minimize_ipopt` works without Jacobian."""
    from scipy.optimize import rosen
    x0 = [1.3, 0.7, 0.8, 1.9, 1.2]
    options = {"tol": 1e-7}
    res = cyipopt.minimize_ipopt(rosen, x0, options=options)

    assert isinstance(res, dict)
    assert np.isclose(res.get("fun"), 0.0)
    assert res.get("status") == 0
    assert res.get("success") is True
    np.testing.assert_allclose(res.get("x"), np.ones(5), rtol=1e-5)


@pytest.mark.skipif("scipy" not in sys.modules,
                    reason="Test only valid if Scipy available.")
def test_minimize_ipopt_nojac_constraints_if_scipy():
    """ `minimize_ipopt` works without Jacobian and with constraints"""
    from scipy.optimize import rosen
    x0 = [1.3, 0.7, 0.8, 1.9, 1.2]
    constr = {"fun": lambda x: rosen(x) - 1.0, "type": "ineq"}
    res = cyipopt.minimize_ipopt(rosen, x0, constraints=constr)
    assert isinstance(res, dict)
    assert np.isclose(res.get("fun"), 1.0)
    assert res.get("status") == 0
    assert res.get("success") is True
    expected_res = np.array([1.001867, 0.99434067, 1.05070075, 1.17906312,
                             1.38103001])
    np.testing.assert_allclose(res.get("x"), expected_res)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
cyipopt: Python wrapper for the Ipopt optimization package, written in Cython.

Copyright (C) 2012-2015 Amit Aides
Copyright (C) 2015-2017 Matthias Kümmerer
Copyright (C) 2017-2021 cyipopt developers

License: EPL 1.0
"""

__version__ = '1.0.3'
